#
# -------------------------------------------------------------------------
# Get link magin data depending on modcod from Ayecka SR1 & format for MRTG
# -------------------------------------------------------------------------
#
#
# Copyright (C) 2014 Francis Breame
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# For details of the GNU General Public License see
# http://perldoc.perl.org/perlgpl.html, or write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#
# Author:	Francis Breame G8ISI
# 		vf0123@btinternet.com
#
#
# 0.1.0	140709	- Initial issue.
# 0.1.1 140710	- Remove reported IP.
#		  Blank negative values.
# 0.1.2	140906	- Report unknown values as UNKNOWN to avoid MTRG errors.
# 0.1.3 140910	- When starting, file is not written until both values have been read 
#		  or 2 mins have elapsed at fast sampling rate. Any existing file will
#		  therefore be maintained.
#

use strict;
no strict "subs";
use warnings;
no warnings "reserved";

use Net::SNMP;
use Time::HiRes qw (usleep);


my ($ip, $rx, $debug, $fn, $fo, $session, $error, $sample, $singleSNMP, $vals, $ntries, $fileTime, $starting, $j);
my (@varbindlist, $modcod, $lastModcod, $lm, $lm8, $lastLm8, $lm16, $lastLm16, $uptime);

my $Q = '"';

my $community	= "public";
my $version	= 2;

my $lmOID		= "1.3.6.1.4.1.27928.101.1.+.4.6.0";
my $modcodStatusOID	= "1.3.6.1.4.1.27928.101.1.+.4.7.0";
my $uptimeOID		= "1.3.6.1.2.1.1.3.0";

my $dvbs2_8psk_3_5	= 12;
my $dvbs2_16apsk_2_3	= 18;

my $startupSample	= 0.5;	# secs
my $normalSample	= 5;

my $startupExit		= 240;	# No of ticks at $startupSample rate after which startup mode is terminated
				# (= 2 min)

my $normalForceWrite	= 300;	# No of secs in normal mode after which file write is forced

my $unknown		= "UNKNOWN";


sub getTS
{
	my ($s,$mi,$h,$d,$m,$y) = localtime(time);
	return sprintf "%04d/%02d/%02d %02d:%02d:%02d", $y+1900,$m+1,$d,$h,$mi,$s;
}

sub debug
{
	return if (!$debug);

	my $s = shift;

	print STDERR getTS()." ".$s;
}

sub init
{
	($session, $error) = Net::SNMP->session(
                           -hostname      => $ip,
                           -version       => $version,
                           -community     => $community 
                        );

	debug("Opened SNMP session to $ip community=$community version=$version error=$error\n") if ($debug);

	undef $modcod;
	undef $lm;
	$lm8 = $lm16 = $lastLm8 = $lastLm16 = "";
	$lastModcod = -1;
	$uptime = $unknown;
	$fileTime = time;
	$ntries = 0;
	$sample = $startupSample;		# Fast rate to start with
	$starting = 1;
}

sub normalise
{
	# Normalise value reported to MRTG
	# 	- $unknown in place of -ve or blank value

	my $n = shift;

	return ($n ne "" && $n>=0) ? $n : $unknown;
}

sub writeFile
{
	open $fo, ">$fn" || die "Can't open output file '$fn'" if (defined $fn); 
				# Reset to overwrite as long it's an actual file 
	print $fo normalise($lm8)."\n".normalise($lm16)."\n$uptime\nAyecka SR1\n";
	close $fo if (defined $fn); 

	debug("File written, age=".(time-$fileTime)."s, lm8=$lm8/$lastLm8, lm16=$lm16/$lastLm16\n");

	$fileTime = time;
}


# Default args
$ip	    = shift;
$rx	    = 1;
$singleSNMP = 0;
$debug	    = 0;

if (!defined $ip || $ip !~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/)
{
	print STDERR "Invalid or missing IP or output file.\n".
		"Usage: (w)perl SR1lm.pl <IP> [-f<output file>] [-r<rx no>] [-s<sample secs>] [-b<n>] [-h<n>] [-o]\n".
		"e.g. (w)perl SR1lm.pl 192.168.0.249 ${Q}-fC:\\MRTG 2\\mib\\SR1lm.txt$Q -s10\n".
		"See installation notes for details.\n\n".
		"Important: perl library Net::SNMP must be installed.\n";
	exit;
}

# Parse args
while (defined ($_=shift))
{
	/^-f/io  && do {s/^-f//io;  $fn = $_;  next};			# Output file
	/^-r/io  && do {s/^-r//io;  $rx = $_;  next};			# RX no
	/^-s/io  && do {s/^-s//io;  $normalSample = $_;  next};		# Sample rate s
	/^-b/io  && do {s/^-b//io;  $dvbs2_8psk_3_5 = $_;  next};	# Basic modcod value
	/^-h/io  && do {s/^-h//io;  $dvbs2_16apsk_2_3 = $_;  next};	# High volume modcod value
	/^-o/io  && do {	    $singleSNMP = 1;  next};		# Single SNMP request mode

	/^-d/io  && do {	    $debug = 1;  next};
	print STDERR "Invalid argument '$_'\n"; 
}

if ($rx<1 || $rx>2)
{
	print STDERR "Invalid rx number - must be 1 or 2. Using 1\n";
	$rx = 1;
}

$fo = *STDOUT;		# Use STDOUT in the absence of a file spec ($fn undef)

debug("Single SNMP request mode\n") if ($singleSNMP);
debug("Basic modcod value = $dvbs2_8psk_3_5 , high volume modcod value = $dvbs2_16apsk_2_3\n");

$modcodStatusOID =~ s/\+/$rx/;
$lmOID		 =~ s/\+/$rx/;
push @varbindlist, ($modcodStatusOID, $lmOID, $uptimeOID);

init();

# Init file only if it doesn't exist
writeFile() if (defined $fn && !-f $fn);

while (1)
{
	if (defined $session)
	{
		if ($singleSNMP)
		{
			$modcod = ${$session->get_request($modcodStatusOID)}{$modcodStatusOID};
			$lm     = ${$session->get_request($lmOID)}{$lmOID};
			$uptime = ${$session->get_request($uptimeOID)}{$uptimeOID};
		}
		else
		{
			$vals = $session->get_request(-varbindlist => \@varbindlist);
			if (defined $vals)
			{
				$modcod = $$vals{$modcodStatusOID};
				$lm     = $$vals{$lmOID};
				$uptime = $$vals{$uptimeOID};
			}
			else
			{
				undef $modcod;
			}
		}

		if (defined $modcod && defined $lm && defined $uptime)
		{
			if ($modcod == $dvbs2_8psk_3_5)
			{
				$lm8 = $lm;
			}
			elsif ($modcod == $dvbs2_16apsk_2_3)
			{
				$lm16 = $lm;
			}

			if ($debug && $modcod >= 0 && $modcod != $lastModcod)
			{
				debug("modcod changed $lastModcod -> $modcod\n");
				$lastModcod = $modcod;
			}

			# In normal mode, write triggered by either lm changing or 5 mins elapsed since last write
			if (!$starting)
			{
				# NB: ne because of null string numbers initially
				if ($lm8 ne $lastLm8 || $lm16 ne $lastLm16 || (time - $fileTime) >= $normalForceWrite)
				{
					writeFile();
					$lastLm8  = $lm8;
					$lastLm16 = $lm16;
				}
			}
			# In startup mode, both lms must have changed or 2 mins elapsed
			# NB: ne because of null string numbers initially
			elsif (($lm8 ne $lastLm8 && $lm16 ne $lastLm16) || ++$ntries > $startupExit)
			{
				writeFile();
				$lastLm8  = $lm8;
				$lastLm16 = $lm16;
				$starting = 0;
				$sample = $normalSample;
				debug("Normal mode entered\n");
			}
		}
		else
		{
			init();		# Keep trying until no errors
		}
	}
	else
	{
		init();		# Keep trying until we get a connection
	}

	usleep($sample * 1000000);
}

